package java.awt;

import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleValue;
import javax.accessibility.AccessibleAction;
import sun.awt.AWTAccessor;
import javax.accessibility.AccessibleContext;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.EventListener;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.peer.MenuItemPeer;
import java.awt.event.ActionListener;
import javax.accessibility.Accessible;

public class MenuItem extends MenuComponent implements Accessible
{
    boolean enabled;
    String label;
    String actionCommand;
    long eventMask;
    transient ActionListener actionListener;
    private MenuShortcut shortcut;
    private static final String base = "menuitem";
    private static int nameCounter;
    private static final long serialVersionUID = -21757335363267194L;
    private int menuItemSerializedDataVersion;
    
    public MenuItem() throws HeadlessException {
        this("", null);
    }
    
    public MenuItem(String s) throws HeadlessException {
        this(s, null);
    }
    
    public MenuItem(String label, MenuShortcut shortcut) throws HeadlessException {
        this.enabled = true;
        this.shortcut = null;
        this.menuItemSerializedDataVersion = 1;
        this.label = label;
        this.shortcut = shortcut;
    }
    
    @Override
    String constructComponentName() {
        synchronized (MenuItem.class) {
            return "menuitem" + MenuItem.nameCounter++;
        }
    }
    
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            if (this.peer == null) {
                this.peer = Toolkit.getDefaultToolkit().createMenuItem(this);
            }
        }
    }
    
    public String getLabel() {
        return this.label;
    }
    
    public synchronized void setLabel(String s) {
        this.label = s;
        MenuItemPeer menuItemPeer = (MenuItemPeer)this.peer;
        if (menuItemPeer != null) {
            menuItemPeer.setLabel(s);
        }
    }
    
    public boolean isEnabled() {
        return this.enabled;
    }
    
    public synchronized void setEnabled(boolean b) {
        this.enable(b);
    }
    
    @Deprecated
    public synchronized void enable() {
        this.enabled = true;
        MenuItemPeer menuItemPeer = (MenuItemPeer)this.peer;
        if (menuItemPeer != null) {
            menuItemPeer.setEnabled(true);
        }
    }
    
    @Deprecated
    public void enable(boolean b) {
        if (b) {
            this.enable();
        }
        else {
            this.disable();
        }
    }
    
    @Deprecated
    public synchronized void disable() {
        this.enabled = false;
        MenuItemPeer menuItemPeer = (MenuItemPeer)this.peer;
        if (menuItemPeer != null) {
            menuItemPeer.setEnabled(false);
        }
    }
    
    public MenuShortcut getShortcut() {
        return this.shortcut;
    }
    
    public void setShortcut(MenuShortcut shortcut) {
        this.shortcut = shortcut;
        MenuItemPeer menuItemPeer = (MenuItemPeer)this.peer;
        if (menuItemPeer != null) {
            menuItemPeer.setLabel(this.label);
        }
    }
    
    public void deleteShortcut() {
        this.shortcut = null;
        MenuItemPeer menuItemPeer = (MenuItemPeer)this.peer;
        if (menuItemPeer != null) {
            menuItemPeer.setLabel(this.label);
        }
    }
    
    void deleteShortcut(MenuShortcut menuShortcut) {
        if (menuShortcut.equals(this.shortcut)) {
            this.shortcut = null;
            MenuItemPeer menuItemPeer = (MenuItemPeer)this.peer;
            if (menuItemPeer != null) {
                menuItemPeer.setLabel(this.label);
            }
        }
    }
    
    void doMenuEvent(long when, int modifiers) {
        Toolkit.getEventQueue().postEvent(new ActionEvent(this, 1001, this.getActionCommand(), when, modifiers));
    }
    
    private final boolean isItemEnabled() {
        if (!this.isEnabled()) {
            return false;
        }
        MenuContainer menuContainer = this.getParent_NoClientCode();
        while (menuContainer instanceof Menu) {
            Menu menu = (Menu)menuContainer;
            if (!menu.isEnabled()) {
                return false;
            }
            menuContainer = menu.getParent_NoClientCode();
            if (menuContainer != null) {
                continue;
            }
            return true;
        }
        return true;
    }
    
    boolean handleShortcut(KeyEvent keyEvent) {
        MenuShortcut o = new MenuShortcut(keyEvent.getKeyCode(), (keyEvent.getModifiers() & 0x1) > 0);
        MenuShortcut o2 = new MenuShortcut(keyEvent.getExtendedKeyCode(), (keyEvent.getModifiers() & 0x1) > 0);
        if ((o.equals(this.shortcut) || o2.equals(this.shortcut)) && this.isItemEnabled()) {
            if (keyEvent.getID() == 401) {
                this.doMenuEvent(keyEvent.getWhen(), keyEvent.getModifiers());
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    MenuItem getShortcutMenuItem(MenuShortcut menuShortcut) {
        return menuShortcut.equals(this.shortcut) ? this : null;
    }
    
    protected final void enableEvents(long n) {
        this.eventMask |= n;
        this.newEventsOnly = true;
    }
    
    protected final void disableEvents(long n) {
        this.eventMask &= ~n;
    }
    
    public void setActionCommand(String actionCommand) {
        this.actionCommand = actionCommand;
    }
    
    public String getActionCommand() {
        return this.getActionCommandImpl();
    }
    
    final String getActionCommandImpl() {
        return (this.actionCommand == null) ? this.label : this.actionCommand;
    }
    
    public synchronized void addActionListener(ActionListener actionListener) {
        if (actionListener == null) {
            return;
        }
        this.actionListener = AWTEventMulticaster.add(this.actionListener, actionListener);
        this.newEventsOnly = true;
        return;
    }
    
    public synchronized void removeActionListener(ActionListener actionListener) {
        if (actionListener == null) {
            return;
        }
        this.actionListener = AWTEventMulticaster.remove(this.actionListener, actionListener);
        return;
    }
    
    public synchronized ActionListener[] getActionListeners() {
        return (ActionListener[])this.getListeners(ActionListener.class);
    }
    
    public <T extends EventListener> T[] getListeners(Class<T> clazz) {
        ActionListener actionListener = null;
        if (clazz == ActionListener.class) {
            actionListener = this.actionListener;
        }
        return AWTEventMulticaster.getListeners(actionListener, clazz);
    }
    
    @Override
    protected void processEvent(AWTEvent awtEvent) {
        if ((awtEvent instanceof ActionEvent)) {
            this.processActionEvent((ActionEvent)awtEvent);
        }
    }
    
    @Override
    boolean eventEnabled(AWTEvent awtEvent) {
        if (awtEvent.id == 1001) {
            return (this.eventMask & 0x80L) != 0x0L || this.actionListener != null;
        }
        return super.eventEnabled(awtEvent);
    }
    
    protected void processActionEvent(ActionEvent actionEvent) {
        ActionListener actionListener = this.actionListener;
        if (actionListener != null) {
            actionListener.actionPerformed(actionEvent);
        }
    }
    
    public String paramString() {
        String s = ",label=" + this.label;
        if (this.shortcut != null) {
            s = s + ",shortcut=" + this.shortcut;
        }
        return super.paramString() + s;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        AWTEventMulticaster.save(objectOutputStream, "actionL", this.actionListener);
        objectOutputStream.writeObject(null);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, HeadlessException {
        objectInputStream.defaultReadObject();
        Object object;
        while (null != (object = objectInputStream.readObject())) {
            if ("actionL" == ((String)object).intern()) {
                this.addActionListener((ActionListener)(ActionListener)objectInputStream.readObject());
            }
            else {
                objectInputStream.readObject();
            }
        }
    }
    
    private static native void initIDs();
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new MenuItem.AccessibleAWTMenuItem();
        }
        return this.accessibleContext;
    }
    
    static boolean access$100(MenuItem menuItem) {
        return menuItem.isItemEnabled();
    }
    
    static {
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
        AWTAccessor.setMenuItemAccessor(new AWTAccessor.MenuItemAccessor() {
            @Override
            public boolean isEnabled(MenuItem menuItem) {
                return menuItem.enabled;
            }
            
            @Override
            public String getLabel(MenuItem menuItem) {
                return menuItem.label;
            }
            
            @Override
            public MenuShortcut getShortcut(MenuItem menuItem) {
                return menuItem.shortcut;
            }
            
            @Override
            public String getActionCommandImpl(MenuItem menuItem) {
                return menuItem.getActionCommandImpl();
            }
            
            @Override
            public boolean isItemEnabled(MenuItem menuItem) {
                return MenuItem.access$100(menuItem);
            }
        });
        MenuItem.nameCounter = 0;
    }
    
    protected class AccessibleAWTMenuItem extends MenuComponent.AccessibleAWTMenuComponent implements AccessibleAction, AccessibleValue
    {
        private static final long serialVersionUID = -217847831945965825L;
        
        @Override
        public String getAccessibleName() {
            if (this.accessibleName != null) {
                return this.accessibleName;
            }
            if (MenuItem.this.getLabel() == null) {
                return super.getAccessibleName();
            }
            return MenuItem.this.getLabel();
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.MENU_ITEM;
        }
        
        @Override
        public AccessibleAction getAccessibleAction() {
            return this;
        }
        
        @Override
        public AccessibleValue getAccessibleValue() {
            return this;
        }
        
        @Override
        public int getAccessibleActionCount() {
            return 1;
        }
        
        @Override
        public String getAccessibleActionDescription(int n) {
            if (n == 0) {
                return "click";
            }
            return null;
        }
        
        @Override
        public boolean doAccessibleAction(int n) {
            if (n == 0) {
                Toolkit.getEventQueue().postEvent(new ActionEvent(MenuItem.this, 1001, MenuItem.this.getActionCommand(), EventQueue.getMostRecentEventTime(), 0));
                return true;
            }
            return false;
        }
        
        @Override
        public Number getCurrentAccessibleValue() {
            return Integer.valueOf(0);
        }
        
        @Override
        public boolean setCurrentAccessibleValue(Number n) {
            return false;
        }
        
        @Override
        public Number getMinimumAccessibleValue() {
            return Integer.valueOf(0);
        }
        
        @Override
        public Number getMaximumAccessibleValue() {
            return Integer.valueOf(0);
        }
    }
}
